#!/usr/bin/env python
# coding: utf-8

import os
import pickle

import numpy as np

from typing import Tuple, Dict
from dataclasses import dataclass

from rdstar.utilities.logging import log_to_default


__all__ = [
    "FitInfoForExternalFit",
]


logger = log_to_default(name=__name__)


@dataclass
class FitInfoForExternalFit:
    base_tuples: np.ndarray
    channel_concatenated_tuples: np.ndarray
    normed_tuples: np.ndarray

    sig_ids_by_ordered_component_ids: Dict[int, Tuple[int, ...]]
    ordered_reco_ch_infos: Dict[int, Tuple[int, ...]]
    ordered_observable_cols: Tuple[str, ...]

    bin_edges_by_reco_ch_ids: Dict[int, Tuple[Tuple[float, ...], ...]]

    def __post_init__(self) -> None:
        assert self.channel_concatenated_tuples.shape == self.normed_tuples.shape, (
            self.channel_concatenated_tuples.shape,
            self.normed_tuples.shape,
        )

    def save(self, file_path: str) -> None:
        if os.path.exists(file_path):
            raise FileExistsError(file_path)

        file_dir_name, file_base_name = os.path.split(file_path)
        tmp_file_path = os.path.join(file_dir_name, f"tmp_{file_base_name}")
        assert not os.path.exists(tmp_file_path), tmp_file_path

        with open(tmp_file_path, "wb") as output_file:
            pickle.dump(self, output_file, pickle.HIGHEST_PROTOCOL)
        os.rename(src=tmp_file_path, dst=file_path)

    @staticmethod
    def load(file_path: str) -> "FitInfoForExternalFit":
        if not os.path.exists(file_path):
            raise FileNotFoundError(file_path)

        with open(file_path, "rb") as input_file:
            fit_info: FitInfoForExternalFit = pickle.load(input_file)

        return fit_info
